﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Telerik.Web.Mvc;
using System.ComponentModel;
using Telerik.Web.Mvc.Infrastructure;
using System.Data;

namespace TelerikMvcApplication3
{
    public static class BindingHelper
    {
        public static IQueryable<Dictionary<string, Object>> ApplyFiltering(this IQueryable<Dictionary<string, Object>> data, IList<IFilterDescriptor> filterDescriptors)
        {
            if (filterDescriptors.Any())
            {
                foreach (IFilterDescriptor descriptor in filterDescriptors)
                {
                    if (descriptor is FilterDescriptor)
                    {
                        var filterDescriptor = (FilterDescriptor)descriptor;
                        var member = filterDescriptor.Member;
                        var value = filterDescriptor.Value;

                        switch (filterDescriptor.Operator)
                        {
                            case FilterOperator.EndsWith:
                                data = data.Where(entry => ((string)entry[member]).EndsWith((string)value));
                                break;
                            case FilterOperator.Contains:
                                data = data.Where(entry => ((string)entry[member]).Contains((string)value));
                                break;
                            case FilterOperator.IsEqualTo:
                                data = data.Where(entry => ((string)entry[member]).Equals((string)value));
                                break;
                            case FilterOperator.StartsWith:
                                data = data.Where(entry => ((string)entry[member]).StartsWith((string)value));
                                break;
                            case FilterOperator.IsNotEqualTo:
                                data = data.Where(entry => !((string)entry[member]).Equals((string)value));
                                break;
                        }
                    }
                }
                //data = data.Where(ExpressionBuilder.Expression<Dictionary<string, Object>>(filterDescriptors));
            }
            return data;
        }

        public static IEnumerable ApplyGrouping(this IQueryable<Dictionary<string, Object>> data, IList<GroupDescriptor> groupDescriptors)
        {
            Func<IEnumerable<Dictionary<string, Object>>, IEnumerable<AggregateFunctionsGroup>> selector = null;
            foreach (var group in groupDescriptors.Reverse())
            {
                if (selector == null)
                {

                    selector = orders => BuildInnerGroup(orders, o => o[group.Member]);

                    //selector = orders => BuildInnerGroup(orders, o => o..ContactName);

                    //if (group.Member == "Customer.ContactName")
                    //{
                    //    selector = orders => BuildInnerGroup(orders, o => oCustomer.ContactName);
                    //}
                    //else if (group.Member == "OrderID")
                    //{
                    //    selector = orders => BuildInnerGroup(orders, o => o.OrderID);
                    //}
                    //else if (group.Member == "ShipAddress")
                    //{
                    //    selector = orders => BuildInnerGroup(orders, o => o.ShipAddress);
                    //}
                    //else if (group.Member == "OrderDate")
                    //{
                    //    selector = orders => BuildInnerGroup(orders, o => o.OrderDate);
                    //}                
                }
                else
                {

                    selector = BuildGroup(o => o[group.Member], selector);
                    //if (group.Member == "Customer.ContactName")                    
                    //{                        
                    //    selector = BuildGroup(o => o.Customer.ContactName, selector);
                    //}            
                    //else if (group.Member == "OrderID")                    
                    //{                        
                    //    selector = BuildGroup(o => o.OrderID, selector);
                    //}            
                    //else if (group.Member == "ShipAddress")                    
                    //{                        
                    //    selector = BuildGroup(o => o.ShipAddress, selector);
                    //}                    
                    //else if (group.Member == "OrderDate")                    
                    //{                        
                    //    selector = BuildGroup(o => o.OrderDate, selector);
                    //}            
                }
            }
            return selector.Invoke(data);
        }

        private static Func<IEnumerable<Dictionary<string, Object>>, IEnumerable<AggregateFunctionsGroup>> BuildGroup<T>(Func<Dictionary<string, Object>, T> groupSelector, Func<IEnumerable<Dictionary<string, Object>>, IEnumerable<AggregateFunctionsGroup>> selectorBuilder)
        {
            var tempSelector = selectorBuilder;
            return g => g.GroupBy(groupSelector).Select(c => new AggregateFunctionsGroup
            {
                Key = c.Key,
                HasSubgroups = true,
                Items = tempSelector.Invoke(c).ToList()
            });
        }

        private static IEnumerable<AggregateFunctionsGroup> BuildInnerGroup<T>(IEnumerable<Dictionary<string, Object>> group, Func<Dictionary<string, Object>, T> groupSelector)
        {
            return group.GroupBy(groupSelector).Select(i => new AggregateFunctionsGroup
            {
                Key = i.Key,
                Items = i.ToList()
            });
        }

        public static IQueryable<Dictionary<string, Object>> ApplyPaging(this IQueryable<Dictionary<string, Object>> data, int currentPage, int pageSize)
        {
            if (pageSize > 0 && currentPage > 0)
            {
                data = data.Skip((currentPage - 1) * pageSize);
            }
            data = data.Take(pageSize);
            return data;
        }

        public static IQueryable<Dictionary<string, Object>> ApplySorting(this IQueryable<Dictionary<string, Object>> data, IList<GroupDescriptor> groupDescriptors, IList<SortDescriptor> sortDescriptors)
        {
            if (groupDescriptors.Any())
            {
                foreach (var groupDescriptor in groupDescriptors.Reverse())
                {
                    data = AddSortExpression(data, groupDescriptor.SortDirection, groupDescriptor.Member);
                }
            }
            if (sortDescriptors.Any())
            {
                foreach (SortDescriptor sortDescriptor in sortDescriptors)
                {
                    data = AddSortExpression(data, sortDescriptor.SortDirection, sortDescriptor.Member);
                }
            }
            return data;
        }

        private static IQueryable<Dictionary<string, Object>> AddSortExpression(this IQueryable<Dictionary<string, Object>> data, ListSortDirection sortDirection, string memberName)
        {
            if (sortDirection == ListSortDirection.Ascending)
            {

                data = data.OrderBy(order => order[memberName]);

                //  switch (memberName)
                //{
               // data = data.OrderBy(order => order.Table.Columns[memberName]);

                //case "OrderID":
                //    data = data.OrderBy(order => order.OrderID);
                //    break;
                //case "Customer.ContactName":
                //    data = data.OrderBy(order => order.Customer.ContactName);
                //    break;
                //case "ShipAddress":
                //    data = data.OrderBy(order => order.ShipAddress);
                //    break;
                //case "OrderDate":
                //    data = data.OrderBy(order => order.OrderDate);
                //    break;
                // }            
            }
            else
            {
               // data = data.OrderBy(order => order.Table.Columns[memberName]);

                //data = data.OrderByDescending(order => order.GetType().GetProperty(memberName));

                data = data.OrderByDescending(order => order[memberName]);

                //switch (memberName)                
                //{                    
                //    case "OrderID":                        
                //        data = data.OrderByDescending(order => order.OrderID);
                //        break;
                //    case "Customer.ContactName":                        
                //        data = data.OrderByDescending(order => order.Customer.ContactName);
                //        break;
                //    case "ShipAddress":                        
                //        data = data.OrderByDescending(order => order.ShipAddress);
                //        break;
                //    case "OrderDate":                        
                //        data = data.OrderByDescending(order => order.OrderDate);
                //        break;
                //}            
            }
            return data;
        }
    }
}